@@ -84,6 +84,9 @@ Simple aggregates
 Custom aggregate expressions
 ----------------------------
 
+Aggregate expression can be virtually any SQL expression not just a simple function taking one parameter. You can try things such as subqueries and different kinds of functions.
+
+In the following example we have a Catalog of products where each catalog knows the net worth of its products.
 
 ::
 
@@ -202,6 +205,54 @@ to define lots of relationships pointing to same class, remember to define the r
         customer_id = sa.Column(sa.Integer, sa.ForeignKey(Customer.id))
 
 
+Many-to-Many aggregates
+-----------------------
+
+Aggregate expressions also support many-to-many relationships. The usual use scenarios includes things such as:
+
+1. Friend count of a user
+2. Group count where given user belongs to
+
+::
+
+
+        user_group = sa.Table('user_group', Base.metadata,
+            sa.Column('user_id', sa.Integer, sa.ForeignKey('user.id')),
+            sa.Column('group_id', sa.Integer, sa.ForeignKey('group.id'))
+        )
+
+
+        class User(Base):
+            __tablename__ = 'user'
+            id = sa.Column(sa.Integer, primary_key=True)
+            name = sa.Column(sa.Unicode(255))
+
+            @aggregated('groups', sa.Column(sa.Integer, default=0))
+            def group_count(self):
+                return sa.func.count('1')
+
+            groups = sa.orm.relationship(
+                'Group',
+                backref='users',
+                secondary=user_group
+            )
+
+
+        class Group(Base):
+            __tablename__ = 'group'
+            id = sa.Column(sa.Integer, primary_key=True)
+            name = sa.Column(sa.Unicode(255))
+
+
+
+        user = User(name=u'John Matrix')
+        user.groups = [Group(name=u'Group A'), Group(name=u'Group B')]
+
+        session.add(user)
+        session.commit()
+
+        session.refresh(user)
+        user.group_count  # 2
 
 
 Multi-level aggregates
@@ -294,7 +345,6 @@ Average movie rating
 TODO
 ----
 
-* Support calculation of many-to-many aggregates
 * Special consideration should be given to `deadlocks`_.
 
 
@@ -305,6 +355,7 @@ TODO
 
 
 from collections import defaultdict
+from weakref import WeakKeyDictionary
 
 import sqlalchemy as sa
 import six
@@ -317,6 +368,9 @@ except ImportError:
     from sqlalchemy.sql.expression import _FunctionGenerator
 
 
+aggregated_attrs = WeakKeyDictionary(defaultdict(list))
+
+
 class AggregatedAttribute(declared_attr):
     def __init__(
         self,
@@ -332,12 +386,10 @@ class AggregatedAttribute(declared_attr):
         self.relationship = relationship
 
     def __get__(desc, self, cls):
-        if '__aggregates__' not in cls.__dict__:
-            cls.__aggregates__ = {}
-        cls.__aggregates__[desc.fget.__name__] = {
-            'expression': desc.fget,
-            'relationship': desc.relationship
-        }
+        if cls not in aggregated_attrs:
+            aggregated_attrs[cls] = [(desc.fget, desc.relationship)]
+        else:
+            aggregated_attrs[cls].append((desc.fget, desc.relationship))
         return desc.column
 
 
@@ -384,13 +436,9 @@ class AggregatedValue(object):
             {self.attr: self.aggregate_query}
         )
         if len(self.relationships) == 1:
-            remote_pairs = self.relationships[-1].property.local_remote_pairs
+            prop = self.relationships[-1].property
 
-            return query.where(
-                remote_pairs[0][0].in_(
-                    getattr(obj, remote_pairs[0][1].key) for obj in objects
-                )
-            )
+            return query.where(self.local_condition(prop, objects))
         else:
             # Builds query such as:
             #
@@ -434,13 +482,22 @@ class AggregatedValue(object):
         return from_
 
     def local_condition(self, prop, objects):
-        return prop.local_remote_pairs[0][0].in_(
-            getattr(
-                obj,
-                prop.local_remote_pairs[0][1].key
+        if prop.secondary is not None:
+            return prop.local_remote_pairs[1][0].in_(
+                getattr(
+                    obj,
+                    prop.local_remote_pairs[1][0].key
+                )
+                for obj in objects
+            )
+        else:
+            return prop.local_remote_pairs[0][0].in_(
+                getattr(
+                    obj,
+                    prop.local_remote_pairs[0][1].key
+                )
+                for obj in objects
             )
-            for obj in objects
-        )
 
 
 class AggregationManager(object):
@@ -449,12 +506,11 @@ class AggregationManager(object):
 
     def reset(self):
         self.generator_registry = defaultdict(list)
-        self.pending_queries = defaultdict(list)
 
     def register_listeners(self):
         sa.event.listen(
             sa.orm.mapper,
-            'mapper_configured',
+            'after_configured',
             self.update_generator_registry
         )
         sa.event.listen(
@@ -463,30 +519,30 @@ class AggregationManager(object):
             self.construct_aggregate_queries
         )
 
-    def update_generator_registry(self, mapper, class_):
-        if '__aggregates__' in class_.__dict__:
-            for key, value in six.iteritems(class_.__aggregates__):
+    def update_generator_registry(self):
+        for class_, attrs in six.iteritems(aggregated_attrs):
+            for expr, relationship in attrs:
                 relationships = []
                 rel_class = class_
 
-                for path_name in value['relationship'].split('.'):
+                for path_name in relationship.split('.'):
                     rel = getattr(rel_class, path_name)
                     relationships.append(rel)
                     rel_class = rel.mapper.class_
 
-                self.generator_registry[rel_class.__name__].append(
+                self.generator_registry[rel_class].append(
                     AggregatedValue(
                         class_=class_,
-                        attr=key,
+                        attr=expr.__name__,
                         relationships=list(reversed(relationships)),
-                        expr=value['expression'](class_)
+                        expr=expr(class_)
                     )
                 )
 
     def construct_aggregate_queries(self, session, ctx):
         object_dict = defaultdict(list)
         for obj in session:
-            class_ = obj.__class__.__name__
+            class_ = obj.__class__
             if class_ in self.generator_registry:
                 object_dict[class_].append(obj)
 
